from typing import Optional

from airflow_pydantic import CallablePath, ImportPath, Task, TaskArgs
from pydantic import Field

__all__ = (
    "SkipOperatorArgs",
    "SkipOperator",
    "FailOperatorArgs",
    "FailOperator",
    "PassOperatorArgs",
    "PassOperator",
)


class SkipOperatorArgs(TaskArgs, extra="allow"):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.skip", description="python_callable")


class FailOperatorArgs(TaskArgs, extra="allow"):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.fail", description="python_callable")


class PassOperatorArgs(TaskArgs, extra="allow"):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.pass_", description="python_callable")


class SkipOperator(Task, SkipOperatorArgs):
    operator: ImportPath = Field(default="airflow.operators.python.PythonOperator", description="airflow operator path", validate_default=True)


class FailOperator(Task, FailOperatorArgs):
    operator: ImportPath = Field(default="airflow.operators.python.PythonOperator", description="airflow operator path", validate_default=True)


class PassOperator(Task, PassOperatorArgs):
    operator: ImportPath = Field(default="airflow.operators.python.PythonOperator", description="airflow operator path", validate_default=True)



from pydantic import BaseModel

from .common import Tool

__all__ = ("clone_repo", "GitRepo")


def clone_repo(name: str, repo: str, branch: str = "main", *, install: bool = True, install_deps: bool = False, tool: Tool = "pip"):
    ret = f"""
[[ -d {name} ]] || git clone {repo}
pushd {name}
git stash
git clean -fdx
git fetch --all --force
git checkout {branch}
git reset origin/{branch} --hard
"""
    if install:
        tool = "uv pip" if tool == "uv" else "pip"
        install_deps_flag = "" if install_deps else "--no-deps "
        cmd = f"{tool} install {install_deps_flag}-e ."
        ret += f"\n{cmd}\n"
    return ret


class GitRepo(BaseModel):
    name: str
    repo: str
    branch: str = "main"

    install: bool = True
    install_deps: bool = False
    tool: Tool = "pip"

    def clone(self):
        return clone_repo(name=self.name, repo=self.repo, branch=self.branch, install=self.install, install_deps=self.install_deps, tool=self.tool)



from typing import List

from pydantic import BaseModel

__all__ = ("BashCommands",)


class BashCommands(BaseModel):
    commands: List[str]

    def render(self, quote: str = "'", escape: bool = False, login: bool = True) -> str:
        from airflow_common_operators import in_bash

        return in_bash("\n".join(["set -ex"] + self.commands), quote=quote, escape=escape, login=login)



from unittest.mock import patch

import pytest
from airflow.exceptions import AirflowSkipException

from airflow_common_operators import ping


class TestCommon:
    def test_ping(self):
        assert ping("localhost")()
        with pytest.raises(AirflowSkipException):
            ping("nonexistent")()

    def test_ping_localappend(self):
        with patch("airflow_common_operators.utility.host.call") as call:
            call.return_value = 0
            ping("blerg")()
            assert call.call_args[0][0] == ["ping", "-c", "1", "blerg.local"]



from pydantic import BaseModel as PydanticBaseModel

__all__ = ("BaseModel",)


class BaseModel(PydanticBaseModel):
    @property
    def params(self):
        return self.model_dump(exclude_unset=False)
